@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,19 +14,18 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 3831 2008-04-15 23:35:20Z eitani $"
-__version__   = "$Revision: 3831 $"
-__date__      = "$Date: 2008-04-15 19:35:20 -0400 (Tue, 15 Apr 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: flat_review.py 4364 2008-11-22 21:42:36Z joanied $"
+__version__   = "$Revision: 4364 $"
+__date__      = "$Date: 2008-11-22 16:42:36 -0500 (Sat, 22 Nov 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import re
-import sys
 
 import pyatspi
 import braille
@@ -136,9 +135,14 @@ class Word:
                     [char, startOffset, endOffset] = text.getTextAtOffset(
                         self.startOffset + i,
                         pyatspi.TEXT_BOUNDARY_CHAR)
+                    # Sometimes we get more than a character's worth. See
+                    # Bug #495303. We can try to correct this.
+                    #
+                    if len(char):
+                        char = char.decode("UTF-8")[0].encode("UTF-8")
                     [x, y, width, height] = text.getRangeExtents(
                         startOffset,
-                        endOffset,
+                        startOffset + 1,
                         0)
                     self.chars.append(Char(self,
                                            i,
@@ -569,14 +573,15 @@ class Line:
 
                 brailleOffset += len(region.string.decode("UTF-8"))
 
-            if len(self.brailleRegions):
-                pad = braille.Region(" ")
-                pad.brailleOffset = brailleOffset
-                self.brailleRegions.append(pad)
-                brailleOffset += 1
-            eol = braille.Region("$l")
-            eol.brailleOffset = brailleOffset
-            self.brailleRegions.append(eol)
+            if not settings.disableBrailleEOL:
+                if len(self.brailleRegions):
+                    pad = braille.Region(" ")
+                    pad.brailleOffset = brailleOffset
+                    self.brailleRegions.append(pad)
+                    brailleOffset += 1
+                eol = braille.Region("$l")
+                eol.brailleOffset = brailleOffset
+                self.brailleRegions.append(eol)
 
         return self.brailleRegions
 
@@ -596,11 +601,6 @@ class Context:
     WRAP_TOP_BOTTOM = 1 << 1
     WRAP_ALL        = (WRAP_LINE | WRAP_TOP_BOTTOM)
 
-    # A minimal chunk to jump around should we not really know where we
-    # are going.
-    #
-    GRID_SIZE = 7
-
     def __init__(self, script):
         """Create a new Context that will be used for handling flat
         review mode.
@@ -643,7 +643,7 @@ class Context:
             currentZoneIndex = 0
             while currentZoneIndex < len(line.zones):
                 zone = line.zones[currentZoneIndex]
-                if zone.accessible == searchZone:
+                if self.script.isSameObject(zone.accessible, searchZone):
                     foundZoneWithFocus = True
                     break
                 else:
@@ -729,34 +729,6 @@ class Context:
         #
         self.targetCharInfo = None
 
-
-    def visible(self,
-                ax, ay, awidth, aheight,
-                bx, by, bwidth, bheight):
-        """Returns true if any portion of region 'a' is in region 'b'
-        """
-        highestBottom = min(ay + aheight, by + bheight)
-        lowestTop = max(ay, by)
-
-        leftMostRightEdge = min(ax + awidth, bx + bwidth)
-        rightMostLeftEdge = max(ax, bx)
-
-        visible = False
-
-        if (lowestTop <= highestBottom) \
-           and (rightMostLeftEdge <= leftMostRightEdge):
-            visible = True
-        elif (aheight == 0):
-            if (awidth == 0):
-                visible = (lowestTop == highestBottom) \
-                          and (leftMostRightEdge == rightMostLeftEdge)
-            else:
-                visible = leftMostRightEdge <= rightMostLeftEdge
-        elif (awidth == 0):
-            visible = (lowestTop <= highestBottom)
-
-        return visible
-
     def clip(self,
              ax, ay, awidth, aheight,
              bx, by, bwidth, bheight):
@@ -818,9 +790,9 @@ class Context:
             else:
                 [x, y, width, height] = text.getRangeExtents( \
                           substringStartOffset, substringEndOffset, 0)
-                if self.visible(x, y, width, height,
-                                cliprect.x, cliprect.y,
-                                cliprect.width, cliprect.height):
+                if self.script.visible(x, y, width, height,
+                                       cliprect.x, cliprect.y,
+                                       cliprect.width, cliprect.height):
 
                     anyVisible = True
 
@@ -1093,10 +1065,10 @@ class Context:
         #
         extents = icomponent.getExtents(0)
 
-        if not self.visible(extents.x, extents.y,
-                            extents.width, extents.height,
-                            cliprect.x, cliprect.y,
-                            cliprect.width, cliprect.height):
+        if not self.script.visible(extents.x, extents.y,
+                                   extents.width, extents.height,
+                                   cliprect.x, cliprect.y,
+                                   cliprect.width, cliprect.height):
             return []
 
         debug.println(
@@ -1142,9 +1114,9 @@ class Context:
             [width, height] = iimage.getImageSize()
 
             if (width != 0) and (height != 0) \
-                   and self.visible(x, y, width, height,
-                                    cliprect.x, cliprect.y,
-                                    cliprect.width, cliprect.height):
+                   and self.script.visible(x, y, width, height,
+                                           cliprect.x, cliprect.y,
+                                           cliprect.width, cliprect.height):
 
                 clipping = self.clip(x, y, width, height,
                                      cliprect.x, cliprect.y,
@@ -1192,12 +1164,21 @@ class Context:
             and accessible.childCount > 0:
             pass
         elif len(zones) == 0:
-            if accessible.name and len(accessible.name):
+            string = ""
+            if role == pyatspi.ROLE_COMBO_BOX:
+                try:
+                    selection = accessible[0].querySelection()
+                except:
+                    string = self.script.getDisplayedText(accessible[0])
+                else:
+                    item = selection.getSelectedChild(0)
+                    if item:
+                        string = item.name
+
+            if not string and accessible.name and len(accessible.name):
                 string = accessible.name
             elif accessible.description and len(accessible.description):
                 string = accessible.description
-            else:
-                string = ""
 
             if (string == "") \
                 and (role != pyatspi.ROLE_TABLE_CELL):
@@ -1215,91 +1196,6 @@ class Context:
 
         return zones
 
-    def getShowingDescendants(self, parent):
-        """Given a parent that manages its descendants, return a list of
-        Accessible children that are actually showing.  This algorithm
-        was inspired a little by the srw_elements_from_accessible logic
-        in Gnopernicus, and makes the assumption that the children of
-        an object that manages its descendants are arranged in a row
-        and column format.
-        """
-
-        if not parent:
-            return []
-
-        try:
-            icomponent = parent.queryComponent()
-        except NotImplementedError:
-            return []
-
-        descendants = []
-
-        parentExtents = icomponent.getExtents(0)
-
-        # [[[TODO: WDW - HACK related to GAIL bug where table column
-        # headers seem to be ignored:
-        # http://bugzilla.gnome.org/show_bug.cgi?id=325809.  The
-        # problem is that this causes getAccessibleAtPoint to return
-        # the cell effectively below the real cell at a given point,
-        # making a mess of everything.  So...we just manually add in
-        # showing headers for now.  The remainder of the logic below
-        # accidentally accounts for this offset, yet it should also
-        # work when bug 325809 is fixed.]]]
-        #
-        try:
-            table = parent.queryTable()
-        except NotImplementedError:
-            table = None
-            
-        if table:
-            for i in range(0, table.nColumns):
-                header = table.getColumnHeader(i)
-                if header:
-                    extents = header.queryComponent().getExtents(0)
-                    stateset = header.getState()
-                    if stateset.contains(pyatspi.STATE_SHOWING) \
-                       and (extents.x >= 0) and (extents.y >= 0) \
-                       and (extents.width > 0) and (extents.height > 0) \
-                       and self.visible(extents.x, extents.y,
-                                        extents.width, extents.height,
-                                        parentExtents.x, parentExtents.y,
-                                        parentExtents.width,
-                                        parentExtents.height):
-                        descendants.append(header)
-
-        # This algorithm goes left to right, top to bottom while attempting
-        # to do *some* optimization over queries.  It could definitely be
-        # improved.
-        #
-        currentY = parentExtents.y
-        while currentY < (parentExtents.y + parentExtents.height):
-            currentX = parentExtents.x
-            minHeight = sys.maxint
-            while currentX < (parentExtents.x + parentExtents.width):
-                child = icomponent.getAccessibleAtPoint(currentX,
-                                                            currentY,
-                                                            0)
-                if child:
-                    extents = child.queryComponent().getExtents(0)
-                    if extents.x >= 0 and extents.y >= 0:
-                        newX = extents.x + extents.width
-                        minHeight = min(minHeight, extents.height)
-                        if not descendants.count(child):
-                            descendants.append(child)
-                    else:
-                        newX = currentX + self.GRID_SIZE
-                else:
-                    newX = currentX + self.GRID_SIZE
-                if newX <= currentX:
-                    currentX += self.GRID_SIZE
-                else:
-                    currentX = newX
-            if minHeight == sys.maxint:
-                minHeight = self.GRID_SIZE
-            currentY += minHeight
-
-        return descendants
-
     def getShowingZones(self, root):
         """Returns a list of all interesting, non-intersecting, regions
         that are drawn on the screen.  Each element of the list is the
@@ -1382,11 +1278,11 @@ class Context:
                 zones = self.getZonesFromAccessible(root, rootexts)
         except NotImplementedError:
             pass
-        
-        stateset = root.getState()
-        if stateset.contains(pyatspi.STATE_MANAGES_DESCENDANTS) \
-           and (root.childCount > 50):
-            for child in self.getShowingDescendants(root):
+
+        showingDescendants = \
+            self.script.getShowingDescendants(root)
+        if len(showingDescendants):
+            for child in showingDescendants:
                 zones.extend(self.getShowingZones(child))
         else:
             for i in range(0, root.childCount):
@@ -1563,8 +1459,9 @@ class Context:
             # handle toolkits that will offset the caret position to
             # the right if you click dead on center of a character.
             #
+            x = max(x, x + (width / 2) - 1)
             eventsynthesizer.clickPoint(x,
-                                        y + height/ 2,
+                                        y + height / 2,
                                         button)
         except:
             debug.printException(debug.LEVEL_SEVERE)
