@@ -31,9 +31,9 @@
 NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
 """
 
-__id__        = "$Id: $"
-__version__   = "$Revision: $"
-__date__      = "$Date: 2007-02-28 $"
+__id__ = "$Id: speechdispatcherfactory.py 4365 2008-11-24 15:30:25Z wwalker $"
+__version__   = "$Revision: 4365 $"
+__date__      = "$Date: 2008-11-24 10:30:25 -0500 (Mon, 24 Nov 2008) $"
 __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
@@ -104,7 +104,7 @@ class SpeechServer(speechserver.SpeechServer):
         when it is not possible to create the server.
         
         """
-        if not cls._active_servers.has_key(serverId):
+        if serverId not in cls._active_servers:
             cls(serverId)
         # Don't return the instance, unless it is succesfully added
         # to `_active_Servers'.
